#version 330 core
#ifdef GL_ES
precision mediump float;
precision mediump int;
#endif
uniform sampler2D Texture0;
uniform uint PolyFlags;
uniform float Gamma;

uniform vec4 DrawColor;
uniform struct FogParameters
{
	vec4  FogColor;		// Fog color
	float FogStart;		// Only for linear fog
	float FogEnd;		// Oonly for linear fog
	float FogDensity;	// For exp and exp2 equation
	int FogMode; // 0 = linear, 1 = exp, 2 = exp2
} FogParams;

float getFogFactor(FogParameters Params, float FogCoord)
{
	float FogResult = 0.0;
	if(Params.FogMode == 0)
		FogResult = (Params.FogEnd-FogCoord)/(Params.FogEnd-Params.FogStart);
	else if(Params.FogMode == 1)
		FogResult = exp(-Params.FogDensity*FogCoord);
	else if(Params.FogMode == 2)
		FogResult = exp(-pow(Params.FogDensity*FogCoord, 2.0));

	FogResult = 1.0-clamp(FogResult, 0.0, 1.0);

	return FogResult;
}

in vec2 vTexCoords;
in vec4 vEyeSpacePos;

out vec4 FragColor;

//Unreal 227 PolyFlags. May require edits for other UE1 games. Todo: put as #include.
const uint PF_Invisible			= 0x00000001u;	// Poly is invisible.
const uint PF_Masked			= 0x00000002u;	// Poly should be drawn masked.
const uint PF_Translucent	 	= 0x00000004u;	// Poly is transparent.
const uint PF_NotSolid			= 0x00000008u;	// Poly is not solidu; doesn't block.
const uint PF_Environment   	= 0x00000010u;	// Poly should be drawn environment mapped.
const uint PF_Semisolid	  		= 0x00000020u;	// Poly is semi-solid = collision solidu; Csg nonsolid.
const uint PF_Modulated 		= 0x00000040u;	// Modulation transparency.
const uint PF_FakeBackdrop		= 0x00000080u;	// Poly looks exactly like backdrop.
const uint PF_TwoSided			= 0x00000100u;	// Poly is visible from both sides.
const uint PF_AutoUPan		 	= 0x00000200u;	// Automatically pans in U direction.
const uint PF_AutoVPan 			= 0x00000400u;	// Automatically pans in V direction.
const uint PF_NoSmooth			= 0x00000800u;	// Don't smooth textures.
const uint PF_BigWavy 			= 0x00001000u;	// Poly has a big wavy pattern in it.
const uint PF_SpecialPoly		= 0x00001000u;	// Game-specific poly-level render control (reuse BigWavy flag)
const uint PF_SmallWavy			= 0x00002000u;	// Small wavy pattern (for water/enviro reflection).
const uint PF_Flat				= 0x00004000u;	// Flat surface.
const uint PF_ForceViewZone		= 0x00004000u;	// Force current iViewZone in OccludeBSP (reuse Flat flag)
const uint PF_LowShadowDetail	= 0x00008000u;	// Low detail shadows.
const uint PF_NoMerge			= 0x00010000u;	// Don't merge poly's nodes before lighting when rendering.
const uint PF_AlphaBlend 		= 0x00020000u;	// This poly should be alpha blended
const uint PF_DirtyShadows		= 0x00040000u;	// Dirty shadows.
const uint PF_BrightCorners		= 0x00080000u;	// Brighten convex corners.
const uint PF_SpecialLit		= 0x00100000u;	// Only speciallit lights apply to this poly.
const uint PF_Gouraud			= 0x00200000u;	// Gouraud shaded.
const uint PF_NoBoundRejection	= 0x00200000u;	// Disable bound rejection in OccludeBSP (reuse Gourard flag)
const uint PF_Unlit				= 0x00400000u;	// Unlit.
const uint PF_HighShadowDetail	= 0x00800000u;	// High detail shadows.
const uint PF_Portal			= 0x04000000u;	// Portal between iZones.
const uint PF_Mirrored			= 0x08000000u;	// Reflective surface.

// Editor flags.
const uint PF_Memorized			= 0x01000000u;	// Editor: Poly is remembered.
const uint PF_Selected			= 0x02000000u;	// Editor: Poly is selected.
const uint PF_Highlighted		= 0x10000000u;	// Editor: Poly is highlighted.
const uint PF_FlatShaded		= 0x40000000u;	// FPoly has been split by SplitPolyWithPlane.

// Internal.
const uint PF_EdProcessed 		= 0x40000000u;	// FPoly was already processed in editorBuildFPolys.
const uint PF_EdCut       		= 0x80000000u;	// FPoly has been split by SplitPolyWithPlane.
const uint PF_RenderFog			= 0x40000000u;	// Render with fogmapping.
const uint PF_Occlude			= 0x80000000u;	// Occludes even if const uint PF_NoOcclude.
const uint PF_RenderHint		= 0x01000000u;   // Rendering optimization hint.

void main (void)
{
	vec4 TotalColor = texture(Texture0, vTexCoords); // Surface.Texture

	// Handle PF_Masked.
	if ( (PolyFlags&PF_Masked) == PF_Masked && TotalColor.a < 0.5 )
	{
		TotalColor.rgb /= TotalColor.a;
		discard;
	}
	else if ( (PolyFlags&PF_AlphaBlend) == PF_AlphaBlend && TotalColor.a < 0.01 )
		discard;

	if (DrawColor.x != 0.0 || DrawColor.y !=0.0 || DrawColor.z != 0.0)
		TotalColor *= DrawColor; // Add DrawColor.

	// Add distance fog
	if ((FogParams.FogMode == 0 && FogParams.FogEnd > FogParams.FogStart) || (FogParams.FogMode > 0 && FogParams.FogDensity > 0.0 ))
	{
		float FogCoord = abs(vEyeSpacePos.z/vEyeSpacePos.w);
		TotalColor = mix(TotalColor, FogParams.FogColor, getFogFactor(FogParams, FogCoord));
	}

	// Gamma
	TotalColor.r=pow(TotalColor.r,1.8-Gamma);
	TotalColor.g=pow(TotalColor.g,1.8-Gamma);
	TotalColor.b=pow(TotalColor.b,1.8-Gamma);

	FragColor = TotalColor;
}
